import { Kind } from 'graphql/language';
import { GraphQLScalarType } from 'graphql';

const json = {
  name: 'Json',
  description:
    'The JSON scalar type represents JSON values as specified by [ECMA-404](http://www.ecma-international.org/ publications/files/ECMA-ST/ECMA-404.pdf).',
  serialize: value => value,
  parseValue: value => value,
  parseLiteral: (ast) => {
    switch (ast.kind) {
      case Kind.STRING:
      case Kind.BOOLEAN:
        return ast.value;
      case Kind.INT:
      case Kind.FLOAT:
        return parseFloat(ast.value);
      case Kind.OBJECT: {
        const value = Object.create(null);
        ast.fields.forEach((field) => {
          value[field.name.value] = json.parseLiteral(field.value);
        });

        return value;
      }
      case Kind.LIST:
        return ast.values.map(json.parseLiteral);
      default:
        return null;
    }
  },
};

export default {
  schema: `
    scalar Json
  `,
  resolvers: {
    Json: new GraphQLScalarType(json),
  },
};
